@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2009, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2010, Live Networks, Inc.  All rights reserved
 //	Help by Carlo Bonamico to get working for Windows
 // Delay queue
 // Implementation
@@ -33,7 +33,7 @@ int Timeval::operator>=(const Timeval& arg2) const {
 
 void Timeval::operator+=(const DelayInterval& arg2) {
   secs() += arg2.seconds(); usecs() += arg2.useconds();
-  if (usecs() >= MILLION) {
+  if (useconds() >= MILLION) {
     usecs() -= MILLION;
     ++secs();
   }
@@ -41,23 +41,24 @@ void Timeval::operator+=(const DelayInterval& arg2) {
 
 void Timeval::operator-=(const DelayInterval& arg2) {
   secs() -= arg2.seconds(); usecs() -= arg2.useconds();
-  if (usecs() < 0) {
+  if ((int)useconds() < 0) {
     usecs() += MILLION;
     --secs();
   }
-  if (secs() < 0)
+  if ((int)seconds() < 0)
     secs() = usecs() = 0;
+
 }
 
 DelayInterval operator-(const Timeval& arg1, const Timeval& arg2) {
   time_base_seconds secs = arg1.seconds() - arg2.seconds();
   time_base_seconds usecs = arg1.useconds() - arg2.useconds();
 
-  if (usecs < 0) {
+  if ((int)usecs < 0) {
     usecs += MILLION;
     --secs;
   }
-  if (secs < 0)
+  if ((int)secs < 0)
     return DELAY_ZERO;
   else
     return DelayInterval(secs, usecs);
